==> core-1.0.14.tgz: package/lib/form.js <==
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { LICENSE_STATE, PACKAGE_LICENSE_URL, costructorTypes, elementConstructors, licensePlateClass, licensePlateStyle, } from './constants.js';
import { getFormElementType, getLicenseText, handleInvalidLicenseLog, mountElement, objectToFormData, processLicenseKey, setLicenseKey, transformFieldName, usesLicensedFetures, } from './utils.js';
export default class Form {
    constructor(parentId, options) {
        this.options = {
            id: '',
            saveProgress: false,
            useFormData: false,
            schema: [],
            action: null,
            method: null,
            className: 'formjs-form',
        };
        this._parent = null;
        this._formElement = null;
        this._id = '';
        this._isValid = null;
        this._errors = [];
        this._fields = {};
        this._groups = {};
        this._buttons = {};
        this._data = {};
        this._saveProgress = false;
        this._licenseState = LICENSE_STATE.INVALID;
        this._schema = [];
        this._dataPrefixMap = {};
        this.initializeOptions(options);
        this.setParentElement(parentId);
        this.processLicense();
        this.onGui();
        this.initForm();
    }
    setParentElement(parentId) {
        this._parent = document.getElementById(parentId);
        if (!this._parent)
            throw new Error('Parent does not exist in the document');
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, options);
        this._id = options.id;
        this._schema = options.schema;
        this._saveProgress = options.saveProgress;
    }
    initForm() {
        if (this._formElement)
            this.buildSchema(this._schema, this._formElement);
        if (this._formElement && this._parent)
            mountElement(this._formElement, this._parent);
    }
    processLicense() {
        if (!usesLicensedFetures())
            return;
        if (this.options.licenseKey)
            setLicenseKey(this.options.licenseKey);
        this._licenseState = processLicenseKey();
        if (!this.hasValidLicense()) {
            handleInvalidLicenseLog(this._licenseState);
        }
    }
    buildSchema(schema, parent, groupId = null, listId = null, key = null) {
        schema.forEach((options) => {
            this.buildElement(options, parent, groupId, listId, key);
        });
    }
    isFormValid() {
        return this._isValid;
    }
    createWrapper(parent) {
        const wrapper = document.createElement('div');
        mountElement(wrapper, parent);
        return wrapper;
    }
    mapFieldToDataPrefix(options, id, key = null) {
        let newId = options.id;
        if (key) {
            newId = transformFieldName(id, key, options.id);
        }
        this._dataPrefixMap[newId] = {
            id,
            dataKey: options.id,
            key,
        };
        if (key) {
            options.id = newId;
            options.name = newId;
        }
    }
    buildElement(options, parent, groupId, listId, key) {
        const duplicatedOptions = Object.assign({}, options);
        const Constructor = elementConstructors[options.type];
        if (!Constructor)
            throw new Error(`Unknown type: ${options.type}`);
        const wrapper = this.createWrapper(parent);
        const formElementType = getFormElementType(options.type);
        if (listId && key && formElementType === costructorTypes.field) {
            this.mapFieldToDataPrefix(options, listId, key);
        }
        if (groupId && formElementType === costructorTypes.field) {
            this.mapFieldToDataPrefix(options, groupId, null);
        }
        const Constructed = new Constructor(wrapper, this, duplicatedOptions);
        switch (formElementType) {
            case costructorTypes.button:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._buttons[options.id] = Constructed;
                break;
            case costructorTypes.group:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._groups[options.id] = Constructed;
                if (options.schema && Constructed.getSchemaContainer) {
                    const newParent = Constructed.getSchemaContainer();
                    if (newParent) {
                        this.buildSchema(options.schema, newParent, options.prefixSchema ? Constructed.getId() : groupId, listId, key);
                    }
                }
                break;
            case costructorTypes.field:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else {
                    this._fields[options.id] = Constructed;
                }
                break;
        }
    }
    assignToListField(listId, key, constructed, formElementType, id) {
        const list = this._fields[listId];
        if (!list || !list.assignButton || !list.assignGroup || !list.assignField)
            return;
        switch (formElementType) {
            case costructorTypes.button:
                list.assignButton(id, key, constructed);
                break;
            case costructorTypes.group:
                list.assignGroup(id, key, constructed);
                break;
            case costructorTypes.field:
                list.assignField(id, key, constructed);
                break;
        }
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            Object.keys(this._groups).forEach((key) => {
                this._groups[key].update();
            });
            Object.keys(this._fields).forEach((key) => {
                this._fields[key].update();
            });
            Object.keys(this._buttons).forEach((key) => {
                this._buttons[key].update();
            });
        });
    }
    getField(id) {
        return this._fields[id];
    }
    getFormElement() {
        return this._formElement;
    }
    setData(id, value) {
        if (this._dataPrefixMap.hasOwnProperty(id)) {
            this.setDataFromMap(id, value);
        }
        else {
            this.setSimpleData(id, value);
        }
        this.update();
    }
    setDataFromMap(id, value) {
        const fieldMapping = this._dataPrefixMap[id];
        const { dataKey, key, id: prefixDataId } = fieldMapping;
        if (!dataKey || !prefixDataId)
            return;
        if (key) {
            const list = this.getField(prefixDataId);
            if (!list || !list.getKeyIndex)
                return;
            const prefixId = list.getId();
            const keyIndex = list.getKeyIndex(key);
            this.ensureDataStructureExists(prefixId, keyIndex);
            this._data[prefixId][keyIndex][dataKey] = value;
        }
        else {
            const group = this.getGroup(prefixDataId);
            if (!group)
                return;
            const prefixId = group.getId();
            this.ensureDataStructureExists(prefixId);
            this._data[prefixId][dataKey] = value;
        }
    }
    setSimpleData(id, value) {
        this._data[id] = value;
    }
    ensureDataStructureExists(listId, keyIndex = null) {
        if (keyIndex === null && !this._data[listId]) {
            this._data[listId] = {};
            return;
        }
        else if (keyIndex || keyIndex === 0) {
            if (!this._data[listId])
                this._data[listId] = [];
            if (!this._data[listId][keyIndex])
                this._data[listId][keyIndex] = {};
        }
    }
    removeListData(listId, index) {
        if (this._data[listId] && this._data[listId][index]) {
            this._data[listId].splice(index, 1);
        }
    }
    getId() {
        return this._id;
    }
    getData() {
        return this._data;
    }
    getButtons() {
        return this._buttons;
    }
    getButton(id) {
        return this._buttons[id];
    }
    getGroups() {
        return this._groups;
    }
    getGroup(id) {
        return this._groups[id];
    }
    savesProgress() {
        return this._saveProgress;
    }
    hasValidLicense() {
        return this._licenseState === LICENSE_STATE.VALID;
    }
    updateError(id, isValid) {
        if (this._errors.includes(id) && isValid) {
            this._errors = this._errors.filter((field) => {
                return field !== id;
            });
        }
        else if (!this._errors.includes(id) && !isValid) {
            this._errors.push(id);
        }
        this._isValid = this._errors.length === 0;
    }
    reset(event) {
        if (event)
            event.preventDefault();
        Object.keys(this._groups).forEach((key) => {
            const group = this._groups[key];
            if (group.reset && typeof group.reset === 'function')
                group.reset();
        });
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.reset && typeof field.reset === 'function')
                field.reset();
        });
        Object.keys(this._buttons).forEach((key) => {
            const button = this._buttons[key];
            if (button.reset && typeof button.reset === 'function')
                button.reset();
        });
    }
    validate() {
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.validate && typeof field.validate === 'function')
                field.validate();
        });
    }
    save() {
        if (!this._saveProgress || !this.hasValidLicense())
            return;
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.save && typeof field.save === 'function')
                field.save();
        });
        Object.keys(this._groups).forEach((key) => {
            const group = this._groups[key];
            if (group.save && typeof group.save === 'function')
                group.save();
        });
    }
    load() {
        if (!this._saveProgress || !this.hasValidLicense())
            return;
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.load && typeof field.load === 'function')
                field.load();
        });
        Object.keys(this._groups).forEach((key) => {
            const group = this._groups[key];
            if (group.load && typeof group.load === 'function')
                group.load();
        });
    }
    onGui() {
        this._formElement = document.createElement('form');
        this._formElement.setAttribute('id', this._id);
        this._formElement.className = this.options.className;
        if (this.options.action)
            this._formElement.setAttribute('action', this.options.action);
        if (this.options.method)
            this._formElement.setAttribute('method', this.options.method);
        this._formElement.addEventListener('submit', (event) => this.submit(event, this));
        this._formElement.addEventListener('reset', this.reset);
        if (usesLicensedFetures() && !this.hasValidLicense())
            this.createInvalidElement();
    }
    createInvalidElement() {
        const invalid = document.createElement('a');
        invalid.setAttribute('href', PACKAGE_LICENSE_URL);
        invalid.className = licensePlateClass;
        invalid.style.cssText = licensePlateStyle;
        const text = document.createElement('span');
        text.innerText = getLicenseText(this._licenseState);
        mountElement(text, invalid);
        if (this._parent)
            mountElement(invalid, this._parent);
    }
    submit(event, form) {
        event.preventDefault();
        form.validate();
        if (!form.isFormValid()) {
            return;
        }
        if (this.options.action) {
            const element = this.getFormElement();
            if (element)
                element.submit();
        }
        let data = form.getData();
        if (this.options.useFormData) {
            data = objectToFormData(form.getData());
        }
        if (form.options.submit)
            form.options.submit(data);
    }
}

==> core-1.1.0.tgz: package/lib/form.js <==
import { ACTION_ATTRIBUTE, DIV_ELEMENT, FORM_CLASS_DEFAULT, FORM_ELEMENT, HREF_ATTRIBUTE, ID_ATTRIBUTE, LICENSE_STATE, LINK_ELEMENT, METHOD_ATTRIBUTE, PACKAGE_LICENSE_URL, RESET_ATTRIBUTE, SPAN_ELEMENT, SUBMIT_ATTRIBUTE, constructorTypes, elementConstructors, licensePlateClass, licensePlateStyle, } from './constants';
import { getFormElementType, getLicenseText, handleInvalidLicenseLog, mountElement, objectToFormData, processLicenseKey, setLicenseKey, transformFieldName, usesLicensedFetures, } from './utils';
export class Form {
    options = {
        id: '',
        saveProgress: false,
        useFormData: false,
        schema: [],
        action: null,
        method: null,
        className: FORM_CLASS_DEFAULT,
    };
    _parent = null;
    _formElement = null;
    _id = '';
    _isValid = null;
    _errors = [];
    _fields = {};
    _groups = {};
    _buttons = {};
    _data = {};
    _saveProgress = false;
    _licenseState = LICENSE_STATE.INVALID;
    _schema = [];
    _dataPrefixMap = {};
    constructor(parent, options) {
        this.initializeOptions(options);
        this.setParentElement(parent);
        this.processLicense();
        this.onGui();
        this.initForm();
    }
    setParentElement(parent) {
        let parentElement;
        if (typeof parent === 'string') {
            parentElement = document.getElementById(parent);
        }
        else {
            parentElement = parent;
        }
        this._parent = parentElement;
        if (!this._parent)
            throw new Error('Parent does not exist in the document');
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, options);
        this._id = options.id;
        this._schema = options.schema;
        this._saveProgress = options.saveProgress;
    }
    initForm() {
        if (this._formElement)
            this.buildSchema(this._schema, this._formElement);
        if (this._formElement && this._parent)
            mountElement(this._formElement, this._parent);
    }
    processLicense() {
        if (!usesLicensedFetures())
            return;
        if (this.options.licenseKey)
            setLicenseKey(this.options.licenseKey);
        this._licenseState = processLicenseKey();
        if (!this.hasValidLicense()) {
            handleInvalidLicenseLog(this._licenseState);
        }
    }
    buildSchema(schema, parent, groupId = null, listId = null, key = null) {
        schema.forEach((options) => {
            this.buildElement(options, parent, groupId, listId, key);
        });
    }
    isValid() {
        return this._isValid;
    }
    createWrapper(parent) {
        const wrapper = document.createElement(DIV_ELEMENT);
        mountElement(wrapper, parent);
        return wrapper;
    }
    mapFieldToDataPrefix(options, id, key = null) {
        let newId = options.id;
        if (key) {
            newId = transformFieldName(id, key, options.id);
        }
        this._dataPrefixMap[newId] = {
            id,
            dataKey: options.id,
            key,
        };
        if (key) {
            options.id = newId;
            options.name = newId;
        }
    }
    buildElement(options, parent, groupId, listId, key) {
        const duplicatedOptions = {
            ...options,
        };
        const Constructor = elementConstructors[options.type];
        if (!Constructor)
            throw new Error(`Unknown type: ${options.type}`);
        const wrapper = this.createWrapper(parent);
        const formElementType = getFormElementType(options.type);
        if (listId && key && formElementType === constructorTypes.field) {
            this.mapFieldToDataPrefix(options, listId, key);
        }
        if (groupId && formElementType === constructorTypes.field) {
            this.mapFieldToDataPrefix(options, groupId, null);
        }
        const Constructed = new Constructor(wrapper, this, duplicatedOptions);
        switch (formElementType) {
            case constructorTypes.button:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._buttons[options.id] = Constructed;
                break;
            case constructorTypes.group:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else
                    this._groups[options.id] = Constructed;
                if (options.schema && Constructed.getSchemaContainer) {
                    const newParent = Constructed.getSchemaContainer();
                    if (newParent) {
                        this.buildSchema(options.schema, newParent, options.prefixSchema ? Constructed.getId() : groupId, listId, key);
                    }
                }
                break;
            case constructorTypes.field:
                if (listId && key) {
                    this.assignToListField(listId, key, Constructed, formElementType, options.id);
                }
                else {
                    this._fields[options.id] = Constructed;
                }
                break;
        }
    }
    assignToListField(listId, key, constructed, formElementType, id) {
        const list = this._fields[listId];
        if (!list || !list.assignButton || !list.assignGroup || !list.assignField)
            return;
        switch (formElementType) {
            case constructorTypes.button:
                list.assignButton(id, key, constructed);
                break;
            case constructorTypes.group:
                list.assignGroup(id, key, constructed);
                break;
            case constructorTypes.field:
                list.assignField(id, key, constructed);
                break;
        }
    }
    async update() {
        Object.keys(this._groups).forEach((key) => {
            this._groups[key].update();
        });
        Object.keys(this._fields).forEach((key) => {
            this._fields[key].update();
        });
        Object.keys(this._buttons).forEach((key) => {
            this._buttons[key].update();
        });
    }
    getField(id) {
        return this._fields[id];
    }
    getFormElement() {
        return this._formElement;
    }
    setData(id, value) {
        if (Object.prototype.hasOwnProperty.call(this._dataPrefixMap, id)) {
            this.setDataFromMap(id, value);
        }
        else {
            this.setSimpleData(id, value);
        }
        this.update();
    }
    removeData(id) {
        if (Object.prototype.hasOwnProperty.call(this._dataPrefixMap, id)) {
            this.removeMappedData(id);
        }
        else {
            delete this._data[id];
        }
    }
    removeMappedData(id) {
        const fieldMapping = this._dataPrefixMap[id];
        const { dataKey, key, id: prefixDataId } = fieldMapping;
        if (!dataKey || !prefixDataId)
            return;
        if (key) {
            const list = this.getField(prefixDataId);
            if (!list || !list.getKeyIndex)
                return;
            const prefixId = list.getId();
            const keyIndex = list.getKeyIndex(key);
            delete this._data[prefixId][keyIndex][dataKey];
            if (typeof this._data[prefixId][keyIndex] === 'object' &&
                Object.keys(this._data[prefixId][keyIndex]).length === 0) {
                delete this._data[prefixId][keyIndex];
            }
        }
        else {
            const group = this.getGroup(prefixDataId);
            if (!group)
                return;
            const prefixId = group.getId();
            delete this._data[prefixId][dataKey];
            if (typeof this._data[prefixId] === 'object' && Object.keys(this._data[prefixId]).length === 0) {
                delete this._data[prefixId];
            }
        }
    }
    setDataFromMap(id, value) {
        const fieldMapping = this._dataPrefixMap[id];
        const { dataKey, key, id: prefixDataId } = fieldMapping;
        if (!dataKey || !prefixDataId)
            return;
        if (key) {
            const list = this.getField(prefixDataId);
            if (!list || !list.getKeyIndex)
                return;
            const prefixId = list.getId();
            const keyIndex = list.getKeyIndex(key);
            this.ensureDataStructureExists(prefixId, keyIndex);
            this._data[prefixId][keyIndex][dataKey] = value;
        }
        else {
            const group = this.getGroup(prefixDataId);
            if (!group)
                return;
            const prefixId = group.getId();
            this.ensureDataStructureExists(prefixId);
            this._data[prefixId][dataKey] = value;
        }
    }
    setSimpleData(id, value) {
        this._data[id] = value;
    }
    ensureDataStructureExists(listId, keyIndex = null) {
        if (keyIndex === null && !this._data[listId]) {
            this._data[listId] = {};
            return;
        }
        else if (keyIndex || keyIndex === 0) {
            if (!this._data[listId])
                this._data[listId] = [];
            if (!this._data[listId][keyIndex])
                this._data[listId][keyIndex] = {};
        }
    }
    removeListData(listId, index) {
        if (this._data[listId] && this._data[listId][index]) {
            this._data[listId].splice(index, 1);
        }
    }
    getId() {
        return this._id;
    }
    getData() {
        return this._data;
    }
    getButtons() {
        return this._buttons;
    }
    getButton(id) {
        return this._buttons[id];
    }
    getGroups() {
        return this._groups;
    }
    getGroup(id) {
        return this._groups[id];
    }
    savesProgress() {
        return this._saveProgress;
    }
    hasValidLicense() {
        return this._licenseState === LICENSE_STATE.VALID;
    }
    updateError(id, isValid) {
        if (this._errors.includes(id) && isValid) {
            this._errors = this._errors.filter((field) => {
                return field !== id;
            });
        }
        else if (!this._errors.includes(id) && !isValid) {
            this._errors.push(id);
        }
        this._isValid = this._errors.length === 0;
    }
    reset(event) {
        if (event)
            event.preventDefault();
        Object.keys(this._groups).forEach((key) => {
            const group = this._groups[key];
            if (group.reset && typeof group.reset === 'function')
                group.reset();
        });
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.reset && typeof field.reset === 'function')
                field.reset();
        });
        Object.keys(this._buttons).forEach((key) => {
            const button = this._buttons[key];
            if (button.reset && typeof button.reset === 'function')
                button.reset();
        });
    }
    validate() {
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.validate && typeof field.validate === 'function')
                field.validate();
        });
    }
    save() {
        if (!this._saveProgress || !this.hasValidLicense())
            return;
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.save && typeof field.save === 'function')
                field.save();
        });
        Object.keys(this._groups).forEach((key) => {
            const group = this._groups[key];
            if (group.save && typeof group.save === 'function')
                group.save();
        });
    }
    load() {
        if (!this._saveProgress || !this.hasValidLicense())
            return;
        Object.keys(this._fields).forEach((key) => {
            const field = this._fields[key];
            if (field.load && typeof field.load === 'function')
                field.load();
        });
        Object.keys(this._groups).forEach((key) => {
            const group = this._groups[key];
            if (group.load && typeof group.load === 'function')
                group.load();
        });
    }
    onGui() {
        this._formElement = document.createElement(FORM_ELEMENT);
        this._formElement.setAttribute(ID_ATTRIBUTE, this._id);
        this._formElement.className = this.options.className;
        if (this.options.action)
            this._formElement.setAttribute(ACTION_ATTRIBUTE, this.options.action);
        if (this.options.method)
            this._formElement.setAttribute(METHOD_ATTRIBUTE, this.options.method);
        this._formElement.addEventListener(SUBMIT_ATTRIBUTE, (event) => this.submit(event, this));
        this._formElement.addEventListener(RESET_ATTRIBUTE, this.reset);
        if (usesLicensedFetures() && !this.hasValidLicense())
            this.createInvalidElement();
    }
    createInvalidElement() {
        const invalid = document.createElement(LINK_ELEMENT);
        invalid.setAttribute(HREF_ATTRIBUTE, PACKAGE_LICENSE_URL);
        invalid.className = licensePlateClass;
        invalid.style.cssText = licensePlateStyle;
        const text = document.createElement(SPAN_ELEMENT);
        text.innerText = getLicenseText(this._licenseState);
        mountElement(text, invalid);
        if (this._parent)
            mountElement(invalid, this._parent);
    }
    submit(event, form) {
        event.preventDefault();
        form.validate();
        if (!form.isValid()) {
            return;
        }
        if (this.options.action) {
            const element = this.getFormElement();
            if (element)
                element.submit();
        }
        let data = form.getData();
        if (this.options.useFormData) {
            data = objectToFormData(form.getData());
        }
        if (form.options.submit)
            form.options.submit(data);
    }
    destroy() {
        if (this._parent)
            this._parent.remove();
    }
}
